/*
  Copyright 2011-2016 David Robillard <http://drobilla.net>

  Permission to use, copy, modify, and/or distribute this software for any
  purpose with or without fee is hereby granted, provided that the above
  copyright notice and this permission notice appear in all copies.

  THIS SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef ZIX_AMPATREE_H
#define ZIX_AMPATREE_H

#include "zix/common.h"

/**
   @addtogroup zix
   @{
   @name AMPatree
   @{
*/

typedef struct _ZixAMPatree ZixAMPatree;

/**
   Construct a new Patree.
*/
ZIX_API ZixAMPatree*
zix_ampatree_new(void);

/**
   Destroy `t`.
*/
ZIX_API void
zix_ampatree_free(ZixAMPatree* t);

/**
   Print a DOT description of `t` to `fd`.
*/
ZIX_API void
zix_ampatree_print_dot(const ZixAMPatree* t, FILE* fd);

/**
   Insert `str` into `t`.
*/
ZIX_API ZixStatus
zix_ampatree_insert(ZixAMPatree* t, const char* str, size_t len);

/**
   Search for `str` in `t`.
*/
ZIX_API ZixStatus
zix_ampatree_find(const ZixAMPatree* t, const char* str, const char** match);

/**
   @}
   @}
*/

#endif  /* ZIX_AMPATREE_H */
